! f2py.py -c -m MDlib MDlib.f90 --fcompiler=gnu95 --compiler=mingw32

subroutine CalcEnergy(Pos, q1, Q, M, L, rc, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
	integer, intent(in), dimension(0:NAtom-1) :: q1
	integer, intent(in), dimension(0:NAtom-1, 0:NAtom-1) :: Q
	real(8), intent(in), dimension(0:Dim-1) :: L
    real(8), intent(in) :: rc
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 3000., r0 = 1.
    real(8), dimension(0:Dim-1) :: rij, Posi
	real(8) :: d, d2, id2, id6, id12, id3, id9, z
	real(8) :: rc2, Shift, ShiftE
	integer :: i, j, w
    PEnergy = 0.
	Shift = -4.*(rc**(-12)-rc**(-6))
	rc2=rc*rc
	ShiftE = Exp(- rc/4.3)/rc

    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
		z= Posi(2)
		
		!interaction w/ upper plate
		d = abs(L(2)/2. - z)
		id3 = 1 / (d * d * d)
		id9 = id3 * id3 * id3
		PEnergy = PEnergy + (1./5.) * (id9)
		
		!interaction w/ lower plate
		d = abs(L(2)/2. + z)
		id3 = 1 / (d * d * d)
		id9 = id3 * id3 * id3
		PEnergy = PEnergy + (1./5.) * (id9)

		!electrostatic interaction w/ plates
		PEnergy = PEnergy + q1(i)*z		
		
        !interaction btw peptides
		do j = i + 1, NAtom - 1
			rij = Pos(j,:) - Posi
			! boundary conditions in x,y directions
			rij(0) = rij(0) - L(0)*dnint(rij(0)/L(0))
			rij(1) = rij(1) - L(1)*dnint(rij(1)/L(1))
			d2 = sum(rij * rij)
			d = sqrt(d2)
			if (j==i+1 .and. mod(j,M) >0) then
				! this is a bonded interaction;
				! use the harmonic bond potential here
				PEnergy = PEnergy +0.5*k*(d -r0)**2
            else
				! this is a nonbonded interaction;
				! use the Lennard-Jones potential here
				if (d2 > rc2) then
					cycle
				endif
				id2 = 1. / d2            !inverse squared distance
				id6 = id2 * id2 * id2    !inverse sixth distance
				id12 = id6 * id6         !inverse twelvth distance
				PEnergy = PEnergy + 4. * (id12 - id6) + Shift ! Lennard_Jones Interaction
				! Electrostatic interaction in dilute electrolyte solution
				! debye length = 4.3 nm for 5mM KCl solution in water, at 25'C
				! non-dimensionalize to sigma = 10 A 
				PEnergy = PEnergy + Q(i,j) * (Exp(- d/4.3)/d + ShiftE)
			endif
        enddo
    enddo
end subroutine


subroutine CalcEnergyForces(Pos, q1, Q, M, L, rc, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
	integer, intent(in), dimension(0:NAtom-1) :: q1
	integer, intent(in), dimension(0:NAtom-1, 0:NAtom-1) :: Q
	real(8), intent(in), dimension(0:Dim-1) :: L
    real(8), intent(in) :: rc
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out) :: Forces
	real(8), parameter :: k = 3000., r0 = 1.
    real(8), dimension(0:Dim-1) :: rij, Fij, Fiw, Posi
	real(8) :: d, d2, id2, id6, id12, id3, id9, z
	real(8) :: rc2, Shift, ShiftE
	integer :: i, j, w
    PEnergy = 0.
    Forces = 0.
	Shift = -4.*(rc**(-12)-rc**(-6))
	ShiftE = Exp(- rc/4.3)/rc
	rc2=rc*rc
    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
		z= Posi(2)
		
		!interaction w/ upper plate
		d = abs(L(2)/2. - z)
		id3 = 1 / (d * d * d)
		id9 = id3 * id3 * id3
		PEnergy = PEnergy + (1./5.) * (id9)
		Forces(i,2) = Forces(i,2) - 9.*id9/d
		
		!interaction w/ lower plate
		d = abs(L(2)/2. + z)
		id3 = 1 / (d * d * d)
		id9 = id3 * id3 * id3
		PEnergy = PEnergy + (1./5.) * (id9) 
		Forces(i,2) = Forces(i,2) + 9.*id9/d

		!electrostatic interaction w/ plates
		PEnergy = PEnergy + q1(i)*z	
		Forces(i,2) = Forces(i,2) -q1(i)
	
        do j = i + 1, NAtom - 1
			rij = Pos(j,:) - Posi
			! boundary conditions in x,y directions
			rij(0) = rij(0) - L(0)*dnint(rij(0)/L(0))
			rij(1) = rij(1) - L(1)*dnint(rij(1)/L(1))
			d2 = sum(rij * rij)
			d = sqrt(d2)
			if (j==i+1 .and. mod(j,M) >0) then
				! this is a bonded interaction;
				! use the harmonic bond potential here
				PEnergy = PEnergy +0.5*k*(d -r0)**2
				Fij = k*rij*(1-r0/d)
				Forces(i,:) = Forces(i,:) + Fij
				Forces(j,:) = Forces(j,:) - Fij
            else
				! this is a nonbonded interaction;
				! use the Lennard-Jones potential here
				if (d2 > rc2) then
					cycle
				endif
				id2 = 1. / d2            !inverse squared distance
				id6 = id2 * id2 * id2    !inverse sixth distance
				id12 = id6 * id6         !inverse twelvth distance
				PEnergy = PEnergy + 4. * (id12 - id6) + Shift
				PEnergy = PEnergy + Q(i,j) * (Exp(- d/4.3)/d + ShiftE)
				Fij = rij * (((-48. * id12 + 24. * id6) * id2) -Q(i,j) * (Exp(- d/4.3)/d)*(-1/4.3-1/d))
				Forces(i,:) = Forces(i,:) + Fij
				Forces(j,:) = Forces(j,:) - Fij
			endif
        enddo
    enddo
end subroutine

subroutine PEplates(Pos, M, L, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
	real(8), intent(in), dimension(0:Dim-1) :: L
    real(8), intent(out) :: PEnergy
    real(8), dimension(0:Dim-1) :: Posi
	real(8) :: d, id3, id9, z
	integer :: i
    PEnergy = 0.

    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
		z= Posi(2)
		!interaction w/ upper plate
		d = abs(L(2)/2. - z)
		id3 = 1 / (d * d * d)
		id9 = id3 * id3 * id3
		PEnergy = PEnergy + (1./5.) * (id9)
		!interaction w/ lower plate
		d = abs(L(2)/2. + z)
		id3 = 1 / (d * d * d)
		id9 = id3 * id3 * id3
		PEnergy = PEnergy + (1./5.) * (id9)
    enddo
end subroutine		

subroutine VVIntegrate(Pos, q1, Q, Vel, Accel, M, L, rc, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in) :: rc, dt
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
	integer, intent(in), dimension(0:NAtom-1) :: q1
	integer, intent(in), dimension(0:NAtom-1, 0:NAtom-1) :: Q
	real(8), intent(in), dimension(0:Dim-1) :: L
!f2py intent(in,out) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, q1, Q, M, L, rc, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine