import numpy as np
import glob
import time
import matplotlib.pyplot as plt
from matplotlib.font_manager import FontProperties
import matplotlib.text as text
start=time.time()
####### WHAM #######

# set variables & initialize lists, arrays
Niter = 10 		# maximum number of iterations
nblock = 4
nt=int((30000-1)/nblock)		# the length of the simulation block
tol = 0.05
kbT = 0.4
s=""
Flist= []

# get inputs
files=glob.glob('results*.txt')
Dlist=[2.1,2.2,2.3,2.4,2.5,2.7,3.0,3.2,3.5,4.0,5.0,6.0,8.0]
whamfiles=['']*len(Dlist)
for file in files:
	D=float(file[7:10])
	for i, dist in enumerate(Dlist): 
		if dist == D :
			whamfiles[i]=file

########### WHAM #############
for nb in range(nblock):
	Nwind = len(whamfiles) 		# number of windows
	ebf=np.ones(Nwind)			# exp(beta*fk)
	ebf2=np.ones(Nwind)			# buffer for exp(beta*fk)
	ebw=np.zeros([Nwind,nt,Nwind])		# exp(-beta*Wk) 

	# build ebw array
	k=0
	for file in whamfiles:
		f=open(file,'r')
		lines = f.readlines()[nt*nb:nt*(nb+1)]
		i=0
		for file in whamfiles:
			l=0						# l=observ. index
			for line in lines:
				ebw[i,l,k]=np.exp(-float(line.split()[2]))
				l+=1
			i+=1			# i=umbrella index
		k+=1				# k=umbrella index

	for n in range(Niter):
		for k in range(Nwind):
			ebfk=0.
			for i in range(Nwind):
				for l in range(nt):						# loop over time series
					bottom=0;
					for j in range(Nwind):
						bottom+=ebw[i,l,j]*ebf[j]		# compute the denominator
					ebfk+=ebw[i,l,k]/(nt*bottom)		# sum the integrand
			ebf[k]=1/ebfk						
		print ebf
		
		# test convergence
		conv=1											
		printdelta=[]
		for k in range(Nwind):
			delta=abs(np.log(ebf[k]/ebf2[k]))		# compare new and old free energies
			printdelta.append(delta)
			if delta>=tol : conv=0						# test convergence
			ebf2[k]=ebf[k]

		if conv>0 : break
		if n == Niter-1 : print "maximum iteration reached"

	# save the result (fk)
	F = kbT*np.log(ebf)
	F -= F[-1]
	Flist.append(F)
	s+='nb='+str(nb)+'\n'
	for i in range(Nwind) :
		s+="%-4.1f \t %-12.8f \t %-12.8f \n" %(Dlist[i], F[i], printdelta[i])


# outputs
f=open('FreeE.txt','w')
f.write(s)
f.close()

fig=plt.figure(figsize=(3,2.5))
plt.subplot(111)
plt.errorbar(Dlist,np.average(Flist, axis=0), yerr=np.std(Flist, axis=0),fmt='.-',ecolor='g')
plt.xlabel('distance (nm)')
plt.ylabel('Free Energy (kcal/mol)')

font=FontProperties()
font.set_family('serif')
font.set_size(7)	
for o in fig.findobj(text.Text):
	o.set_fontproperties(font)
		
fig.savefig("FreeEnergy.png",dpi=600, bbox_inches='tight', pad_inches=0.1)
	
end=time.time()-start
print "time="+str(end)